#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Aug  5 17:23:52 2017

@author: yunhuip
"""
import os
import numpy as np
import re
import pandas as pd
from shutil import copyfile
##input mandatory and optional parameters for the simulation
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("receptor", type=str,
                    help="The receptor protein which the target will bind with.")
parser.add_argument("target", type=str,
                    help="The target ligand which will bind with the receptor protein")
parser.add_argument("-l","--scale_low", type=float, default = 0.2,
                    help="The lower boundary in the force scaling. unit in kcal/mol/Angstrom. 1 kcal/mol/Angstrom = 69.479 pN\
                          default = 0.2 kcal/mol/Angstrom")
parser.add_argument("-u","--scale_up", type=float, default = 1.18,
                    help="The upper boundary in the force scaling. unit in kcal/mol/Angstrom. 1 kcal/mol/Angstrom = 69.479 pN\
                          default = 1.18 kcal/mol/Angstrom")
parser.add_argument("-n","--cycle_number", type=int, default = 8000,
                    help="The number of Delphiforce calculation and updating cycles in the simulation.default = 8000")
parser.add_argument("-f","--force_updating_frequency", type=int, default = 500,
                    help="The simulation running steps for each cycle. The smaller running steps make the steered\
                    forces uppdated more frequently. default = 500")
parser.add_argument("-c","--cut_off", type=int, default = 18,
                    help="The cut-off distance used in GBIS MD simulation. default = 18 Angstroms")
parser.add_argument("-d","--langevin_Damping", type=int, default = 1,
                    help="Langevin Damping coefficient used in GBIS MD simulation. default = 1")
parser.add_argument("-m", "--simulation_mode", type=int, choices=[1, 2, 3],default=1,
                    help="""Acceptable simulation mode: 1: Accelerated modeling. The DelPhiForces are calculated and applied as steered forces
                        constantly until the end of simulation; Accelerated modeling is the default simulation mode.
                        2: Cut-off modeling. the DelPhiFoces are calculated and applied as steered forces 
                        only when ligand atoms are far away from the receptor (exceed the  cut-off distance in MD 
                        simulation).                                                                          
                        3: Scaled modeling. When ligands atoms are within the cut-off distance in MD simulation,
                        the steered forces are corrected as the substratcion of the Delphiforce and MD forces to 
                        avoid double counting of the electrostatic interactions in the simulation.
                        """)
parser.add_argument("-p","--path_to_parameters_files", type=str, default = '/common1/compbio/SOFTWARE/DelphiForce_MD/source/',
                    help="""path to the NAMD, Amber Tools and Delphiforce executable files and MD paramteres files.These files are orginally in SOURCE folder. If the path to SOURCE folder is changed, new path should be provided.""")
parser.add_argument("-v","--path_to_vmd", type=str, default = '/common1/compbio/TOOLS/vmd-1.9.3/bin/vmd',
                    help="""path to the VMD executables files. DelphiForce MD require Visual molecular dynamics(VMD) for trajecory analysis. The path to VMD executables should be provided.""")

parser.add_argument("-a","--path_to_amber", type=str, default = '/software/amber/14/bin/',
                    help="""path to the Amber Tools executables files. DelphiForce MD require ambpdb and tleap from Amber tools to generate amber parameters file and pqr file. The path to Amber Tools  executables should be provided.""")

parser.add_argument("-np","--numer_processor", type=int, default = 8,
                    help="""Number of processor core to run NAMD for the steer MD simulation, which depends on the number of processor core in your local machine.""")

args = parser.parse_args()
receptor = args.receptor
target = args.target
scale_low = args.scale_low
scale_up = args.scale_up
cycle_number = args.cycle_number
force_updating_frequency = args.force_updating_frequency
cut_off = args.cut_off
langevin_Damping = args.langevin_Damping
simulation_mode = args.simulation_mode
path_to_parameters_files = args.path_to_parameters_files
vmd = args.path_to_vmd
path_to_amber = args.path_to_amber
numer_processor = args.numer_processor
#%%Prepare Flag and PQR files using input_PDB
##module to add chian ID when the information is missing in the pdb and pqr file
def add_chain_ID(input,chain_ID):
    os.system("grep \"ATOM\" "+input+" > input.pdb")
    f=open("input.pdb")
    pqr=f.readlines()
    f.close
    nlines=len(pqr)
    for j in range(0,nlines):
        pqr_ID=pqr[j][0:21] + str(chain_ID) + pqr[j][22:79]
        f=open(input[0:-4]+'_ID'+input[-4:],'a')
        f.write(pqr_ID)
        f.close()
## Module to set beta and occupancy to be zero for PDB, which will be used for the generating the flag file for steer forces in MD 
def set_zero(input):
    os.system("grep \"ATOM\" "+input+" > input.pdb")
    f=open("input.pdb")
    pdb=f.readlines()
    f.close
    nlines=len(pdb)
    for j in range(0,nlines):
        pdb_zero=pdb[j][0:26] +"       0.000   0.000   0.000  0.00  0.00"  + pdb[j][66:79]
        f=open(input[0:-4]+'.flag','a')
        f.write(pdb_zero)
        f.close()
## Module to extract the forces from namd using TCL force, which will be used to get the forces difference between namd force and Delphiforce in Scaled modeling  
def namd_force():
    copyfile(path_to_parameters_files + 'force_complex.namd',\
    'force_complex.namd')
    copyfile(path_to_parameters_files + 'force_target.namd',\
    'force_target.namd')
    os.system("mkdir out")
##get the total atom numbers for receptor and target
    f=open("receptor_ID.pdb")
    pdb_receptor = f.readlines()
    f.close
    nlines_receptor = len(pdb_receptor)
    f=open("target_ID.pdb")
    pdb_target = f.readlines()
    f.close
    nlines_target = len(pdb_target) 
## specify the atom index of target where the force will be calculated in tcl force
    com_atom_index_start = nlines_receptor + 1
    com_atom_index_end = nlines_receptor + nlines_target
    
    f=open(path_to_parameters_files + 'force_complex.namd')
    config_MD = f.readlines()
    f.close
    for i in range(0,len(config_MD)):
        config_MD[i] = re.sub('atom_index_start', str(com_atom_index_start), config_MD[i])
        config_MD[i] = re.sub('atom_index_end', str(com_atom_index_end), config_MD[i])
        config_MD[i] = re.sub('cut_off', str(cut_off), config_MD[i])
        config_MD[i] = re.sub('langevin_Damping', str(langevin_Damping), config_MD[i])
    f = open("force_complex.namd", 'w')
    f.writelines(config_MD)
    f.close()
    
    f=open(path_to_parameters_files + 'force_target.namd')
    config_MD = f.readlines()
    f.close
    for i in range(0,len(config_MD)):
        config_MD[i] = re.sub('atom_index_start', "1", config_MD[i])
        config_MD[i] = re.sub('atom_index_end', str(nlines_target), config_MD[i])
        config_MD[i] = re.sub('cut_off', str(cut_off), config_MD[i])
        config_MD[i] = re.sub('langevin_Damping', str(langevin_Damping), config_MD[i])
    f = open("force_target.namd", 'w')
    f.writelines(config_MD)
    f.close()

    os.system(path_to_parameters_files + 'namd2 +p ' + str(numer_processor) + ' \
    force_target.namd | grep \"TCL: force:\" | awk \'{print $3,$4,$5}\'> ./force_target.out') 

    os.system(path_to_parameters_files + 'namd2 +p ' + str(numer_processor) + ' \
    force_complex.namd | grep \"TCL: force:\" | awk \'{print $3,$4,$5}\'> ./force_complex.out') 
#%%    
## Module to generate the flag file used for input the forces in the steered MD simulation
def generate_flag_file(input_receptor, input_target, input_scale_low, input_scale_up, input_simulation_mode):
##delete temperatory files
    os.system("grep \"ATOM\" "+input_receptor+" > input_receptor.pdb")
    os.system("grep \"ATOM\" "+input_target+" > input_target.pdb")
    os.system("rm input_steerforce.txt leap_receptor.in leap_target.in receptor_ID.pqr target_ID.pqr \
              receptor_ID.pdb target_ID.pdb receptor_ID.flag target_ID.flag \
              target_ID_delphiforce.flag complex.flag")   
##generate the pqr for receptor
    leap_receptor = ["source leaprc.ff14SB\n",
    "source leaprc.gaff\n",
    "LoadAmberParams " + path_to_parameters_files + "/parameters/parm99.dat\n",
    "sys = loadpdb input_receptor.pdb\n",
    "saveAmberParm sys receptor.prmtop receptor.prmcrd\n",
    "quit\n"]
    f = open("leap_receptor.in", 'w')
    f.writelines(leap_receptor)
    f.close()
    os.system(path_to_amber +'tleap -s -f \
    leap_receptor.in\ncat receptor.prmcrd | ' + path_to_amber + "ambpdb -p receptor.prmtop -pqr > receptor.pqr\n \
    cat receptor.prmcrd | " + path_to_amber  + 'ambpdb -p receptor.prmtop > receptor.pdb\n')
    add_chain_ID("receptor.pqr","R")
    add_chain_ID("receptor.pdb","R")
    set_zero("receptor_ID.pdb")
##generate the pqr for target   
    leap_target = ["source leaprc.ff14SB\n",
    "source leaprc.gaff\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/parm99.dat\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/gaff.dat\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/" + target[0:-4] + ".frcmod\n",
    "LoadAmberPrep " + path_to_parameters_files + "parameters/" + target[0:-4] + "_resp_gaff.prepi\n",
    "sys = loadpdb input_target.pdb\n",
    "saveAmberParm sys target.prmtop target.prmcrd\n",
    "quit\n"]
    f = open("leap_target.in", 'w')
    f.writelines(leap_target)
    f.close()
    os.system(path_to_amber + 'tleap -s -f \
    leap_target.in\ncat target.prmcrd | ' + path_to_amber + 'ambpdb -p target.prmtop -pqr > target.pqr\n \
    cat target.prmcrd | ' + path_to_amber + 'ambpdb -p target.prmtop > target.pdb\n')
    add_chain_ID("target.pqr","T")
    add_chain_ID("target.pdb","T")
    set_zero("target_ID.pdb")
##generate the amber prarmater files  for complex
    os.system('cat input_receptor.pdb input_target.pdb | grep "ATOM" > input_complex.pdb')     ## generate input complex pdb file  
    leap_complex = ["source leaprc.ff14SB\n",
    "source leaprc.gaff\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/parm99.dat\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/gaff.dat\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/" + target[0:-4] + ".frcmod\n",
    "LoadAmberPrep " + path_to_parameters_files + "parameters/" + target[0:-4] + "_resp_gaff.prepi\n",
    "sys = loadpdb input_complex.pdb\n",
    "saveAmberParm sys complex.prmtop complex.prmcrd\n",
    "quit\n"]
    f = open("leap_complex.in", 'w')
    f.writelines(leap_complex)
    f.close()
    os.system(path_to_amber + 'tleap -s -f leap_complex.in')

#Run Delphiforce to compute the forces applied on the target by the receptor
    os.system('cp ' + path_to_parameters_files + 'Delphiforce_run/* ./')
    os.system('./DelPhiForce.sh receptor_ID.pqr '+'target_ID.pqr'+' out')
##Scaling of the Delphiforce output, unit convert to kcal/mol from kt
    f=open('out.atom')
    out=f.readlines()
    f.close
    Delphi_force_out = out[12:len(out)-3]
    Delphi_force = [i.split()[10:13] for i in Delphi_force_out]
    Delphi_force = np.array(Delphi_force,dtype=float)*0.592
##steered forces using Delphiforce or forces differences based on user choice
##Default is not performing force subsratctions
    if  input_simulation_mode == 3:
        namd_force()
        force_target = pd.read_csv('force_target.out',sep=" ",header=None)
        force_complex = pd.read_csv('force_complex.out',sep=" ",header=None)
        namdforce = np.array(force_complex,dtype=float)-np.array(force_target,dtype=float)
        input_steerforce = Delphi_force - namdforce 
    else:
        input_steerforce = Delphi_force
    np.savetxt("input_steerforce.txt", input_steerforce)
    os.system("awk \'{print sqrt($1^2+$2^2+$3^2)}\' input_steerforce.txt > force_magnitude.txt")
    f=open('force_magnitude.txt')
    force_magnitude=f.readlines()
    f.close
    max=float(0.0)
    for i in range(0,len(force_magnitude)):
        if max<float(force_magnitude[i]):
            max=float(force_magnitude[i])
##factor to scale the input force to lower and upper boundary
    factor_scale_low=max/input_scale_low
    factor_scale_up=max/input_scale_up
#print(max)
##Generate Flag file for sMD using Delphiforce output
    f=open('target_ID.flag')
    target_flag=f.readlines()
    f.close
    for j in range(0,len(input_steerforce)):
        Fx=format(float(input_steerforce[j,0]))
        Fy=format(float(input_steerforce[j,1]))
        Fz=format(float(input_steerforce[j,2]))
        if (np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)**2)<0.00001) or (max<0.00001):
            nFx=nFy=nFz='0.000'
            TF='0.00'            
            print(Fx,nFx,Fy,nFy,Fz,nFz,sep=';')
        elif (max>=input_scale_low) and (max<input_scale_up):
            nFx=format(float(Fx)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            nFy=format(float(Fy)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            nFz=format(float(Fz)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            TF=format(abs((np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)**2))),'.2f')

        elif max>=input_scale_up:
            nFx=format(float(Fx)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            nFy=format(float(Fy)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            nFz=format(float(Fz)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            TF=format(abs((np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)**2)))/factor_scale_up,'.2f')

        else:
            nFx=format(float(Fx)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            nFy=format(float(Fy)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            nFz=format(float(Fz)/np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)\
                             **2),'.3f')
            TF=format(abs((np.sqrt(float(Fx)**2+float(Fy)**2+float(Fz)**2))/factor_scale_low),'.2f')           
            print(Fx,nFx,Fy,nFy,Fz,nFz,sep=';')
        len_Fx=len(nFx)
        len_Fy=len(nFy)
        len_Fz=len(nFz)
        Flag_file=target_flag[j][0:(38-len_Fx)]+nFx+target_flag[j]\
        [38:(46-len_Fy)]+nFy+target_flag[j][46:(54-len_Fz)]+nFz+\
        target_flag[j][54:56]+TF+target_flag[j][60:78]
        print(Flag_file)
        f=open('target_ID_delphiforce.flag','a')
        f.write(Flag_file+'\n')
        f.close()
    os.system('cat receptor_ID.flag target_ID_delphiforce.flag > complex.flag') ##generate force flag file
#%%
##Prepare files of initial run of the simulation for minimization of the input structures  
def prepare_sMD_files():
    os.system("cat " + receptor + " "  + target + " | grep \"ATOM\" > complex_initial.pdb")
    leap_complex = ["source leaprc.ff14SB\n",
    "source leaprc.gaff\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/parm99.dat\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/gaff.dat\n",
    "LoadAmberParams " + path_to_parameters_files + "parameters/" + target[0:-4] + ".frcmod\n",
    "LoadAmberPrep " + path_to_parameters_files + "parameters/" + target[0:-4] + "_resp_gaff.prepi\n",
    "sys = loadpdb complex_initial.pdb\n",
    "saveAmberParm sys complex.prmtop complex.prmcrd\n",
    "quit\n"]
    f = open("leap_complex.in", 'w')
    f.writelines(leap_complex)
    f.close()
    os.system(path_to_amber + 'tleap -s -f leap_complex.in')    
    os.system('mkdir out')
    f=open(path_to_parameters_files + 'amber_sMD.conf')
    config_MD = f.readlines()
    f.close
    for i in range(0,len(config_MD)):
        config_MD[i] = re.sub('cut_off', str(cut_off), config_MD[i])
        config_MD[i] = re.sub('langevin_Damping', str(langevin_Damping), config_MD[i])
    f = open("amber_sMD.conf", 'w')
    f.writelines(config_MD)
    f.close()
    os.system("rm leap_complex.in")
#%%
##Prepare sMD resrart files for each cycle
def prepare_sMD_restart_files(configfile,inputname,outputname):
    f=open(path_to_parameters_files + configfile)
    config_MD = f.readlines()
    f.close
    for i in range(0,len(config_MD)):
        config_MD[i] = re.sub('inputname', inputname, config_MD[i])
        config_MD[i] = re.sub('cut_off', str(cut_off), config_MD[i])
        config_MD[i] = re.sub('langevin_Damping', str(langevin_Damping), config_MD[i])
        config_MD[i] = re.sub('force_updating_frequency', str(force_updating_frequency), config_MD[i])
        config_MD[i] = re.sub('outputname_old', outputname, config_MD[i])
        config_MD[i] = re.sub('outputname_new', outputname[0:-len(str(steer_run))]+str(steer_run+1), config_MD[i])
    f = open("amber_sMD_restart.conf", 'w')
    f.writelines(config_MD)
    f.close() 
#%%
#Module to extract the last frame for lastest sMD run 
def get_last_frame(inputname):
    vmd_tcl=[
    "set resid_receptor [ ]\n",
    "set resid_target   [ ]\n",
    "mol load pdb " + receptor + "\n",
    "set sel [atomselect top all]\n",
    "set chains_receptor [lsort -unique [$sel get chain]]\n",
    "foreach chain_receptor $chains_receptor {\n",
    "set sel [atomselect top \"chain $chain_receptor\"]\n",
    "set resid_number [llength [lsort -unique [$sel get resid]]]\n",
    "lappend resid_receptor $resid_number\n",
    "unset  resid_number sel\n",
    "}\n",   
    "mol delete all\n",
    "mol load pdb " + target + "\n",
    "set sel [atomselect top all]\n",
    "set chains_target [lsort -unique [$sel get chain]]\n",
    "foreach chain_target $chains_target {\n",
    "set sel [atomselect top \"chain $chain_target\"]\n",
    "set resid_number [llength [lsort -unique [$sel get resid]]]\n",
    "lappend resid_target $resid_number\n",
    "unset resid_number sel\n",
    "}\n",
    "mol delete all\n",
    "mol load parm7 complex.prmtop dcd ./out/"+inputname+".dcd\n",
    "set end 0\n",
    "foreach chain_receptor $chains_receptor resid_receptor $resid_receptor {\n",
    "set end [expr $end + $resid_receptor]\n",
    "set chains [atomselect top \"resid [expr $end - $resid_receptor +1] to $end\" frame last]\n",
    "$chains set chain $chain_receptor\n",
    "}\n",
    "set receptor_residue_num $end\n",
    "set sel [atomselect top \"resid 1 to $receptor_residue_num\" frame last]\n",
    "$sel writepdb " +inputname + "_receptor.pdb\n",
    "unset sel\n",
    "foreach chain_target $chains_target resid_target $resid_target {\n",
    "set end [expr $end + $resid_target]\n",
    "set chains [atomselect top \"resid [expr $end - $resid_target +1] to $end\" frame last]\n",
    "$chains set chain $chain_target\n",
    "}\n",
    "set sel [atomselect top \"resid [expr $receptor_residue_num +1] to $end\" frame last]\n",
    "$sel writepdb " +inputname + "_target.pdb\n",
    "exit"]
    f = open("getframe_"+inputname+".tcl", 'w')
    f.writelines(vmd_tcl)
    f.close()
    os.system(vmd + ' -dispdev text -e '+"getframe_"+inputname+'.tcl')
#%%
##module to check the ditance between the target atoms and receptor atoms
def distance_check(inputname,mode):
    vmd_tcl=[
    "set resid_receptor [ ]\n",
    "set resid_target   [ ]\n",
    "mol load pdb " + receptor + "\n",
    "set sel [atomselect top all]\n",
    "set chains_receptor [lsort -unique [$sel get chain]]\n",
    "foreach chain_receptor $chains_receptor {\n",
    "set sel [atomselect top \"chain $chain_receptor\"]\n",
    "set resid_number [llength [lsort -unique [$sel get resid]]]\n",
    "lappend resid_receptor $resid_number\n",
    "unset  resid_number sel\n",
    "}\n",
    "mol delete all\n",
    "mol load pdb " + target + "\n",
    "set sel [atomselect top all]\n",
    "set chains_target [lsort -unique [$sel get chain]]\n",
    "foreach chain_target $chains_target {\n",
    "set sel [atomselect top \"chain $chain_target\"]\n",
    "set resid_number [llength [lsort -unique [$sel get resid]]]\n",
    "lappend resid_target $resid_number\n",
    "unset resid_number sel\n",
    "}\n",
    "mol delete all\n",
    "set end 0\n",
    "foreach chain_receptor $chains_receptor resid_receptor $resid_receptor {\n",
    "set end [expr $end + $resid_receptor]\n",
    "}\n",
    "set receptor_residue_num $end\n",
    "foreach chain_target $chains_target resid_target $resid_target {\n",
    "set end [expr $end + $resid_target]\n",
    "}\n",
    "mol load parm7 complex.prmtop dcd ./out/"+inputname+".dcd\n",
    "set sel1 [atomselect top \"(within 4  of  (resid 1 to $receptor_residue_num)) and  (resid [expr $receptor_residue_num +1] to $end)\" frame last]\n",
    "set sel2 [atomselect top \"(within "+ str(cut_off) + " of  (resid 1 to $receptor_residue_num)) and  (resid [expr $receptor_residue_num +1] to $end)\" frame last]\n",
    "set index1 [$sel1 get index]\n",
    "set index2 [$sel2 get index]\n",
    "unset end\n",
    "set outfile [open atom_within_contact.txt w]\n",
    "puts $outfile  \"$index1\"\n",
    "close $outfile\n",
    "set outfile [open atom_within_cut_off.txt w]\n",
    "puts $outfile  \"$index2\"\n",
    "close $outfile\n",
    "exit"]
    f = open("distance_"+inputname+".tcl", 'w')
    f.writelines(vmd_tcl)
    f.close()
    os.system(vmd+' -dispdev text -e '+"distance_"+inputname+'.tcl')
##in cut-off modelling, turn off steered forces for target atoms within MD cut-off distance respect to the receptor
    if mode == 2:
        f=open('atom_within_cut_off.txt')
        atom_within_cutoff = f.readline().split()                               
        f.close 
        f=open('complex.flag')
        flag_file = f.readlines()                               
        f.close
        if atom_within_cutoff != []: 
            for j in range(0,len(atom_within_cutoff)):
                flag_file[int(atom_within_cutoff[j])] = flag_file[int(atom_within_cutoff[j])][0:54]\
                + "  0.00  0.00" +flag_file[int(atom_within_cutoff[j])][66:78]+"\n"
        f = open("complex.flag", 'w')
        f.writelines(flag_file)
        f.close()
##in scaled modelling, turn off steered forces for target atoms which contacts with receptor
    elif mode == 3:
        f=open('atom_within_contact.txt')
        atom_within_contact = f.readline().split()
        f.close
        f=open('complex.flag')
        flag_file = f.readlines()
        f.close
        if atom_within_contact != []:
            for j in range(0,len(atom_within_contact)):
                flag_file[int(atom_within_contact[j])] = flag_file[int(atom_within_contact[j])][0:54]\
                + "  0.00  0.00" +flag_file[int(atom_within_contact[j])][66:78]+"\n"
        f = open("complex.flag", 'w')
        f.writelines(flag_file)
        f.close()
#%%PBS files to run sMD using namd 
def run_sMD(param_file, outputname):
    os.system(path_to_parameters_files + 'namd2 +p ' + str(numer_processor) +' '+param_file+ '> ./out/'\
    +outputname+'.out')
#%%Append DCD files
def dcd_append(dcd_input):
    os.system(path_to_parameters_files + "catdcd -o ./DCD/combine.dcd "
              +"./DCD/tem.dcd "+"./out/"+dcd_input+".dcd")   
#%%Remove TEMP files
def rmove_tmp(name):
    os.system("rm ./"+name+"*")
    os.system("rm ./out/"+name+"*")
    os.system("rm ./getframe_"+name+"*")
    os.system("rm ./distance_"+name+"*")
    os.system("rm ./contact_"+name+"*")
#%%Main perform the sMD
##prepare sMD param files and perform minimization and the first run
os.system('mkdir DCD out')
generate_flag_file(receptor,target,scale_low,scale_up,3)
prepare_sMD_files()
run_sMD('amber_sMD.conf','steer1')
copyfile("./out/steer1.dcd","./DCD/tem.dcd")
steer_run = 1
get_last_frame('steer'+str(steer_run))
distance_check('steer'+str(steer_run),3)
#%% Performing the simulation according to the user-specificed simulation mode
simulation_time = (args.cycle_number*args.force_updating_frequency)/1000000

if args.simulation_mode == 1:
    print("Accelerated modeling is running. The simulation time is {} ns.".format(simulation_time))
    while steer_run <= cycle_number:
        generate_flag_file('steer'+str(steer_run)+'_receptor.pdb',\
                       'steer'+str(steer_run)+'_target.pdb',\
                       scale_low, scale_up,1)
        prepare_sMD_restart_files('amber_sMD_restart.conf', 'complex', 'steer'+str(steer_run))
        steer_run=steer_run+1
        run_sMD('amber_sMD_restart.conf','steer'+str(steer_run))
        dcd_append('steer'+str(steer_run))
        copyfile("./DCD/combine.dcd","./DCD/tem.dcd")
        get_last_frame('steer'+str(steer_run))
        rmove_tmp('steer'+str(steer_run-1))        
        
elif args.simulation_mode == 2:
    print(" Cut-off modeling is running. The simulation time is {} ns.".format(simulation_time))
    while steer_run <= cycle_number:
        generate_flag_file('steer'+str(steer_run)+'_receptor.pdb',\
                       'steer'+str(steer_run)+'_target.pdb',\
                       scale_low, scale_up,1)
        prepare_sMD_restart_files('amber_sMD_restart.conf','complex','steer'+str(steer_run))
        distance_check('steer'+str(steer_run),2)
        steer_run=steer_run+1
        run_sMD('amber_sMD_restart.conf','steer'+str(steer_run))
        dcd_append('steer'+str(steer_run))
        copyfile("./DCD/combine.dcd","./DCD/tem.dcd")
        get_last_frame('steer'+str(steer_run))
        rmove_tmp('steer'+str(steer_run-1))       
else:
    print("Scaled modeling is running. The simulation time is {} ns.".format(simulation_time))
    while steer_run <= cycle_number:
        generate_flag_file('steer'+str(steer_run)+'_receptor.pdb',\
                       'steer'+str(steer_run)+'_target.pdb',\
                       scale_low,scale_up, 3)
        prepare_sMD_restart_files('amber_sMD_restart.conf','complex','steer'+str(steer_run))
        distance_check('steer'+str(steer_run),3)
        steer_run=steer_run+1
        run_sMD('amber_sMD_restart.conf','steer'+str(steer_run))
        dcd_append('steer'+str(steer_run))
        copyfile("./DCD/combine.dcd","./DCD/tem.dcd")
        get_last_frame('steer'+str(steer_run))
        rmove_tmp('steer'+str(steer_run-1))  
        
